* V_TOOL - Common sub-routines
* V0.00 May 7th 1995
* V0.01, pjw, 2022 Jun 17. Bugfixed load arrays > 32k
*

        section code

        xdef    sentinel
        xdef    check_pars,get_open,close,soft_err,hard_err
        xdef    trap3,trap4,print,printb,save,save_r
        xdef    readb,read,load

        xref    ut_stos         ;win1_asm_util_stos_asm
        xref    l2frtr

*
sentinel
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* input : none
* return: d4  = empty stack pointer
*         a1  = ???
*         no errors
*
        move.l  (a7)+,a1        ;get return address
        move.l  #-1,-(a7)       ;unravel usp to here
        move.l  $58(a6),d4      ;d4 = empty stack pointer
        jmp     (a1)            ;return
*
check_pars
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* input : a3/a5 -> parameter list
* return: d2  = separator after first parameter
*         d7  = number of parameters
*         no errors
*
        move.l  a5,d7
        sub.l   a3,d7
        lsr.l   #3,d7           ;d7 = number of parameters
        cmpi.l  #2,d7           ;less than two pars not allowed
        bcs.s   rts_bp
        subq.l  #1,d7           ;don't count filename
        move.w  0(a6,a3.l),d2   ;first par info
        andi.w  #$70,d2         ;extract following separator
        rts
rts_bp
        moveq   #-15,d0
hard_err
        cmpi.l  #-1,(a7)+       ;unravel usp and
        bne.s   hard_err        ; return to BASIC
        tst.l   d0
        rts
*
get_open
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* input : a3/a5 -> parameter list
*         d3  = key
* return:
*         d0  = error
*         a0  = channel id
*         a3 -> next parameter
*         d1/d3/a1 smashed
*
        movem.l d2-d4/d6/a2,-(a7)
        bsr     ut_stos         ;get filename
        movem.l (a7)+,d2-d4/d6/a2
        bne.s   hard_err        ;oops! go tell BASIC direct
        movea.l a1,a0           ;a0->filename
        moveq   #-1,d1          ;this job
        trap    #4              ;relative to a6
        moveq   #1,d0           ;io.open
        trap    #2
        tst.l   d0
        rts
*
save_r
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* input : d2  = length of file
*         a0  = channel id
*         a1 -> buffer
* return:
*         d0  = 0 or no return
*
        trap    #4              ;relative to a6
*
save
*
        moveq   #$49,d0         ;fs.save
        bra.s   trap3
*
printb
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* Print 6 bytes from buffer
* input : d0.l = first 4 bytes to print
*         d2   = last 2 bytes to print
*         a0   = channel id
* return:
*         d0   = 0 or no return
*         a1  -> buffer
*         d2   = preserved
*
        movea.l (a6),a1         ;get pointer to buffer
        move.l  d0,0(a6,a1.l)
        move.w  d2,4(a6,a1.l)
        moveq   #6,d2           ;bytes to send
print
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* input : d2  = no. bytes to send
*         a0  = channel id
*         a1 -> buffer
* return:
*         d0  = 0 or no return
*
        moveq   #7,d0           ;iob.smul

*
trap4
        trap    #4              ;relative to a6

*
trap3
        movem.l d1/d3/a1,-(a7)
        moveq   #-1,d3          ;timeout
        trap    #3
        movem.l (a7)+,d1/d3/a1

*
tret
        tst.l   d0
        beq.s   c_rts

*
soft_err
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* input : d0  = error number
* return: no errors
*
        move.l  d0,d1
        bsr.s   close

se_loop
        cmpi.l  #-1,(a7)+       ;unravel usp
        bne.s   se_loop
        move.l  d4,$58(a6)      ;set empty stack
        bra     l2frtr

*
close
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* input : a0 -> channel id
* return: d0/a0/a1?  = ?
*
        moveq   #2,d0
        trap    #2
c_rts
        rts

*
readb
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* Read d2 bytes into SB buffer
* input : d2.w = number of bytes to read
*         a0   = channel id
* return:
*         d0   = 0 or no return
*         a1  -> buffer
*         d2/a0  preserved
*         no other registers affected
*
        movea.l (a6),a1         ;a1 -> buffer
        moveq   #3,d0           ;iob.fmul
        bra.s   trap4

*
read
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* Reads d2 bytes into a buffer pointed to by a6+a1
*
* input : d2  = no. bytes to send
*         a0  = channel id
*         a1 -> buffer
* return:
*         d0  = 0 or no return
*         d2/a0-a1 preserved
*         no other registers affected
*
        moveq   #3,d0           ;iob.fmul
        bra.s   trap4

*
reada
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* Reads d2 bytes into a buffer pointed to by a6+a1
* V0.01, pjw, 2022 Jun 17, New! Dont preserve a1!
*
* input : d2.w = no. bytes to send
*         a0   = channel id
*         a1  -> buffer
* return:
*         d0   = 0 or no return
*         d2/a0 preserved
*         a1     updated
*         no other registers affected
*
        moveq   #3,d0           ;iob.fmul
        movem.l d1/d3,-(a7)
        moveq   #-1,d3          ;timeout
        trap    #4
        trap    #3
        movem.l (a7)+,d1/d3
        bra.s   tret

*
load
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* Loads d2 bytes into area pointed to by a6+a1
* V0.01, pjw, 2022 Jun 17, preserve a1 here!
*
* input : d2.l = no. bytes to send
*         a0   = channel id
*         a1  -> buffer
* return:
*         d0  = 0 or no return
*         d2/a0-a1 preserved
*         no other registers affected
*
        movem.l d3/d4/a1,-(sp)
        move.l  d2,d4
        move.l  #$7FFF,d3       ;max number of bytes to load in one go ($8000?)
l_loop
        cmp.l   d4,d3           ;if number of bytes left <= max loadable then
        bge.s   l_all           ; load it all in one go
        move.l  d3,d2           ; else load max
        sub.l   d3,d4           ;  and leave the rest for later
        bsr.s   reada           ;load some
        beq.s   l_loop          ; and go back an get some more
        bra.s   l_quit
l_all
        move.l  d4,d2           ;get the rest
        bsr.s   reada
l_quit
        movem.l (sp)+,d3/d4/a1
        rts

*
        end
