* Parinfo - utility to parse parameter information
* PWITTE July 8th 1995
*
        section code

        xdef    parinfo
*
parinfo
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* This routine pre-processes all relevant parameter information from a given
* parameter in the name table and returns to different addresses depending on
* the kind of parameter. To avoid having to have some 20 different return
* addresses to cater for every combination a simplified scheme is employed and
* only three return addresses are proffered. The type is returned both in d5.w
* and in the condition codes:
*
* The Return Code is added to the return address on the stack providing for
* multiple (3) return addresses:
* Code 0 is returned for end of parameters, null parameter
* Code 2 is returned for set or unset scalar variables, and expressions
* Code 4 is returned for set or unset arrays and substrings
*
* Parameter Info Byte: in msb of d0.w (#n denotes the bit set):
* Code 0: all clear=>end parameters (cc=ZvC), #7=>null parameter (cc=c^z)
* Code 2: all clear=>scalar variable, #0=>expression, #7=>unset
*         #1=>loop index: + #2=>for loop  -  #3=>repeat loop
* Code 4: all clear=>normal array, #0=>sub-array, #7=>unset array
*
* Type word:
* 1 = string, 2 = float, 3 = word integer
*
* Seperator Info Byte:
* 0=none 1=,  2=;  3=\  4=!  5=TO  if #31 set parameter was hashed
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*
* input :
*       a3  -> parameter entry in name table
*
* return: cc = set according to type: N => str, Z => flt, N & Z clear => int
*       d0   = parameter info byte.seperator info byte:
*              d0.w lsb: 0=none 1=,  2=;  3=\  4=!  5=TO
*              d0.w msb: (see above)
*              bit #31 set if parameter hashed
*       d5.w = kind | type: 2 or 3 | 1,2 or 3
*       a2  -> array descriptor if parameter is an array (otherwise preserved)
*       errors: not applicable
*       all other registeres preserved
*
* A suitable harness for making use of this routine could look like the
* following: (This for fetching parameters according to type)
*
*       bsr     parinfo
*       bra.s   nopar       ;00: null parameter (or end of parameters)
*       bra.s   getvar      ;02: get (scalar) parameter
*       bra.s   getarr      ;04: get array
* ...
* nopar
*       bls.s   end_of_parameters
*       bhi.s   null_parameter
* getvar
*       bmi.s   getstr
*       beq.s   getflt
*       bra.s   getint
* getstr
*       tst.w   d0           ;look for unset (non-sbasic)
*       bmi.s   nulstr       ;get a null string
* getstring
*       get a string parameter
* etc for flt & int
*
* getarr
*       bmi.s   arrstr
* ...
* arrstr
*       tst.w   d0           ;unset array?
*       bmi.s   error_bp
*       btst    #7,d0        ;sub-string?
*       bne.s   getstring    ; yes, get it as a string
* ...
*
*
        moveq   #0,d5           ;initialise type info register
        cmpa.l  a3,a5           ;end of parameter list?
        bhi.s   notend          ; no, continue
        rts                     ; yes -> no extra info for brevity!
notend
        move.b  0(a6,a3.l),d0   ;get dimensional info
        lsl.w   #1,d0           ;x 2
        move.w  0(a6,a3.l),d5   ;get type info - same for most subroutines here
        andi.w  #$0f0f,d5       ;remove seperator info (and set cc)
        jmp     table(pc,d0.w)
table
        bra.s   unset
        bra.s   expression      ;expression!? unlikely we'll ever see this here!
        bra.s   scalar
        bra.s   array
        bra.s   null            ;SB proc? very strange, better set null
        bra.s   null            ;SB fn? very strange, better set null
        bra.s   repeat          ;loop index
for                             ;loop index
        moveq   #%00000110,d0   ;flag "loop.for" (6)
        bra.s   set_scalar
repeat
        moveq   #%00001010,d0   ;flag "loop.rep" (10)
        bra.s   set_scalar
unset
        beq.s   null            ;d5=0 => null parameter
        move.w  #%010000000,d0  ;set code for unset
        bra.s   set_scalar
null
        moveq   #3,d5           ;type is undefined, but cc set to mark null
        bra.s   unset_array     ;re-use unset array to denote null
scalar
        moveq   #%00000000,d0   ;assume "normal" scalar
        cmpi.w  #-1,2(a6,a3.l)  ;an expression?
        bne.s   set_scalar      ; no, keep flag
expression
        moveq   #%00000001,d0   ;flag "expression"
        bra.s   set_scalar
array
        addq.l  #4,(a7)         ;set array return address
        movea.l 4(a6,a3.l),a2   ;a2 -> value offset
        adda.l  $28(a6),a2      ;add vv.base
        tst.l   0(a6,a2.l)      ;is array unset? (SBASIC relevence only in
        beq.s   unset_array     ; SuperBASIC unset arrays don't pass inerpreter)
        moveq   #%00000000,d0   ;flag "normal" array
        tst.b   d5              ;sub-array? ($0300)
        beq.s   sub_array
        bra.s   seperator
unset_array
        move.w  #%010000000,d0  ;set bit#7 without going negative
        tst.b   d5              ;unset sub-array?
        bne.s   seperator
sub_array
        addq.w  #%00000001,d0   ;flag "sub-array"
        move.w  #$201,d5        ;pretend it's a string
        bra.s   seperator
set_scalar
        addq.l  #2,(a7)         ;code 2
        andi.w  #$000f,d5
        addi.w  #$200,d5        ;force kind to scalar variable
seperator
        lsl.w   #8,d0           ;move parameter info byte out of the way
        move.b  1(a6,a3.l),d0   ;get seperator info
        andi.b  #$f0,d0
        lsr.b   #4,d0
        bclr    #3,d0           ;get hash bit out of the way
        beq.s   return          ;no hash, skip this
        bset    #31,d0          ;move hash bit so it can be tested with tst.l
return
        cmpi.b  #2,d5           ;set condition codes according to type
        rts
*
        end
