* Return an array to an array parrameter
*
        section code

        xdef    retarrf

        xref    load            ;use iop.fmul unless array is whole file!
*
retarrf
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* Description: loads an array from file to an array parameter
* An array can only be loaded if:
*       a) it has been previously dimensioned
*       b) types are same,
*       c) if a string then last dimension must be same
* If sizes differ use smallest size to load.
* (a) & (b) must be tested for before entry to this routine.
*
* input :  Types must match! Unset arrays could crash the machine!
*       d1.l = size of array parameter
*       d2.l = size of array to load
*       d5.b = type: 1=str, 2=flt, 3=int
*       a0     channel id
*       a1  -> parameter array's 0'th element      }
*       a2  -> parameter array descriptor (no dims) } all addresses a6 relative
*       a4  -> stored array's descriptor           }
*       file position must be at 0'th element of stored array
*
* return: cc = set to error in d0
*       d1.l = number of bytes not loaded
*       d3.l = error:
*               EF parameter array > stored array
*               NC    stored array > parameter array
*               if no "soft" errors d3 remains unchanged
*       errors: EF - file error
*               NC - file error
*               BP - last dimn of string not same
*               if any of the above errors routine returns immediately
*       a0/d5    = preserved
*       a1/a2/a4 = updated
*       all other registeres preserved
*
*
        cmpi.b  #1,d5           ;is it a string array?
        bne.s   notstr
        move.l  d1,-(a7)
        bsr.s   getdims
        move.w  d0,d1
        lea.l   4(a2),a4
        bsr.s   getdims
        cmp.w   d1,d0           ;are max string lengths same?
        beq.s   samelen         ; yep, continue
        addq.l  #4,a7           ; no, clear stack
        move.l  d2,d1           ; remainder is 100%
        moveq   #-15,d0         ; bad parameter
        rts                     ; quit
samelen
        move.l  (a7)+,d1
notstr
        cmp.l   d1,d2           ;get smallest length in d2
        bls.s   noremain
        sub.l   d1,d2           ;get remainder
        exg     d1,d2           ;d1 = remainder
        moveq   #-1,d3          ;flag "not enough room for whole array"
        bra.s   load_array
noremain
        beq.s   samesize
        moveq   #-10,d3         ;flag "more room than data"
samesize
        moveq   #0,d1           ;no remainder
load_array
        bra     load            ;load array into area pointed to by 0(a6,a1)
*
getdims
        move.w  0(a6,a4.l),d0   ;number of dimensions
        lsl.w   #2,d0           ;d0 is offset to last index multiplier
        adda.w  d0,a4           ;add that to descriptor offset
        move.w  -2(a6,a4.l),d0  ;d0=last dimn (taking care of slight overshoot)
        rts
*
        end
